"""WAeUP portal certificates
"""
import grok
from hurry.workflow.interfaces import IWorkflowInfo, IWorkflowState
from zope.component.interfaces import IFactory
from zope.interface import implementedBy
from waeup.interfaces import (ICertificate, ICertificateContainer,
                              ICertificateCourse, ICourse, IDepartment)

class Certificate(grok.Container):
    """A certificate.
    """
    grok.implements(ICertificate)

    def __init__(self, code=u'NA', title=u'Unnamed Certificate',
                 category=None, study_mode=None, start_level=None,
                 end_level=None, application_category=None, m_prefix=u'',
                 max_pass = u''):
        super(Certificate, self).__init__()
        self.code = code
        self.title = title
        self.category = category
        self.study_mode = study_mode
        self.start_level = start_level
        self.end_level = end_level
        self.application_category = application_category
        self.m_prefix = m_prefix
        self.max_pass = max_pass
        # Initialize workflow state...
        self.setReviewState('init')

    def check(self):
        """Set Review State to ``checked``.
        """
        self.setReviewState('check')

    def getReviewState(self):
        return IWorkflowState(self).getState()

    def setReviewState(self, transition_id='check'):
        info = IWorkflowInfo(self)
        info.fireTransition(transition_id)
        return

    def addCourseRef(self, course, level=100, core_or_elective=True):
        """Add a course reference.
        """
        code = "%s_%s" % (course.code, level)
        self[code] = CertificateCourse(course, level, core_or_elective)

    def delCourseRef(self, code):
        """Delete a course denoted by its code.
        """
        if code in self.keys():
            del self[code]
            self._p_changed = True
        return

    review_state = property(getReviewState, setReviewState)


class CertificateFactory(grok.GlobalUtility):
    """A factory for certificates.
    """
    grok.implements(IFactory)
    grok.name(u'waeup.Certificate')
    title = u"Create a new certificate.",
    description = u"This factory instantiates new certificate instances."

    def __call__(self, *args, **kw):
        return Certificate(*args, **kw)

    def getInterfaces(self):
        return implementedBy(Certificate)

class CertificateCourse(grok.Model):
    grok.implements(ICertificateCourse)

    def __init__(self, course=None, level=100, core_or_elective=True):
        self.course = course
        self.level = level
        self.core_or_elective = core_or_elective

    def getCourseCode(self):
        return self.course.code
    
class CertificateCourseFactory(grok.GlobalUtility):
    """A factory for certificate courses.
    """
    grok.implements(IFactory)
    grok.name(u'waeup.CertificateCourse')
    title = u"Create a new certificate course.",
    description = u"This factory instantiates new certificate courses."

    def __call__(self, *args, **kw):
        return CertificateCourse(*args, **kw)

    def getInterfaces(self):
        return implementedBy(CertificateCourse)

@grok.subscribe(ICourse, grok.IObjectRemovedEvent)
def removedCourseHandler(course, event):
    """If a course is deleted, we make sure that also references in a
       certificatecontainer are removed.
    """
    old_parent = event.oldParent
    code = course.code
    # Try to find a nearby certificate container...
    dept = getattr(old_parent, '__parent__', None)
    if not IDepartment.providedBy(dept):
        # Hm, looks like this course was not part of a department...
        return
    certcontainer = dept.certificates
    for cert in certcontainer.values():
        # Loop through all certificatecourses in this certificate...
        if code in cert.keys():
            del cert[code]
            cert._p_changed = True

